@@ -49,7 +49,7 @@ static GObjectClass *series_labels_parent_klass;
  * @GOG_SERIES_LABELS_NEAR_ORIGIN: near origin.
  **/
 
-struct {
+static const struct {
 	char const *label;
 	GogSeriesLabelsPos pos;
 } positions [] =
@@ -755,6 +755,7 @@ gog_data_label_finalize (GObject *obj)
 	GogDataLabel *label = GOG_DATA_LABEL (obj);
 	gog_dataset_finalize (GOG_DATASET (obj));
 	g_free (label->format);
+	g_free (label->separator);
 	go_string_unref (label->element.str);
 	data_label_parent_klass->finalize (obj);
 }
@@ -1241,7 +1242,8 @@ gog_series_labels_update (GogObject *obj)
 						case 0: /* protect from an unexpected string end */
 							break;
 						case 'c':
-							next = GO_IS_DATA (labels->custom_labels[0].data)?
+							next = (GO_IS_DATA (labels->custom_labels[0].data) &&
+							        go_data_get_vector_size (labels->custom_labels[0].data) > i)?
 									go_data_get_vector_string (labels->custom_labels[0].data, i):
 									NULL;
 							if (next) {
@@ -1337,6 +1339,7 @@ gog_series_labels_finalize (GObject *obj)
 			go_string_unref (labels->elements[i].str);
 		g_free (labels->elements);
 	}
+	g_list_free (labels->overrides);
 	series_labels_parent_klass->finalize (obj);
 }
 
@@ -1368,11 +1371,19 @@ role_data_label_post_add (GogObject *parent, GogObject *child)
 		go_styled_object_get_style (GO_STYLED_OBJECT (parent)));
 	lbls->overrides = g_list_insert_sorted (lbls->overrides, child,
 		(GCompareFunc) element_compare);
+
+	lbl->format = g_strdup ("");
+	lbl->default_pos = lbls->default_pos;
+	lbl->allowed_pos = lbls->allowed_pos;
+	lbl->position = lbls->position;
+	lbl->supports_percent = lbls->supports_percent;
+
 	plot = (GogPlot *) gog_object_get_parent_typed (child, GOG_TYPE_PLOT);
 	for (j = 0; j < plot->desc.series.num_dim; j++) {
 		/* FIXME, this might depend upon the series type */
 		switch (plot->desc.series.dim[j].ms_type) {
 		case GOG_MS_DIM_VALUES:
+			g_free (lbl->format);
 			lbl->format = g_strdup_printf ("%%%u", j);
 			j = plot->desc.series.num_dim; /* ensure we exit the loop */
 			break;
@@ -1380,11 +1391,6 @@ role_data_label_post_add (GogObject *parent, GogObject *child)
 			break;
 		}
 	}
-	lbl->format = g_strdup ("");
-	lbl->default_pos = lbls->default_pos;
-	lbl->allowed_pos = lbls->allowed_pos;
-	lbl->position = lbls->position;
-	lbl->supports_percent = lbls->supports_percent;
 }
 
 static void
