@@ -166,7 +166,7 @@ apply_ui_from_file (GtkBuilder *gui, GsfInput *src, const char *uifile,
  * Simple utility to open ui files
  *
  * Since 0.9.6
- * Returns: (transfer full): a new #GtkBuilder or NULL
+ * Returns: (transfer full): a new #GtkBuilder or %NULL
  *
  * @uifile should be one of these:
  *
@@ -268,7 +268,7 @@ go_gtk_builder_load (char const *uifile,
  * Simple utility to open ui files
  *
  * Deprecated: 0.9.6, use go_gtk_builder_load().
- * Returns: (transfer full): a new #GtkBuilder or NULL
+ * Returns: (transfer full): a new #GtkBuilder or %NULL
  *
  * @uifile should be one of these:
  *
@@ -294,7 +294,7 @@ go_gtk_builder_new (char const *uifile,
  * Simple utility to open ui files
  *
  * Since: 0.9.6
- * Returns: (transfer full): a new #GtkBuilder or NULL
+ * Returns: (transfer full): a new #GtkBuilder or %NULL
  *
  * Variant of go_gtk_builder_new that searchs goffice directories
  * for files.
@@ -385,7 +385,7 @@ go_gtk_builder_signal_connect_swapped (GtkBuilder	*gui,
  * Simple wrapper to #gtk_builder_get_object which returns the object
  * as a GtkWidget.
  *
- * Returns: (transfer none): a new #GtkWidget or NULL
+ * Returns: (transfer none): a new #GtkWidget or %NULL
  **/
 GtkWidget *
 go_gtk_builder_get_widget (GtkBuilder *gui, char const *widget_name)
@@ -401,7 +401,7 @@ go_gtk_builder_get_widget (GtkBuilder *gui, char const *widget_name)
  * searches the #GtkComboBox in @gui and ensures it has a model and a
  * renderer appropriate for using with #gtk_combo_box_append_text and friends.
  *
- * Returns: (transfer none): the #GtkComboBox or NULL
+ * Returns: (transfer none): the #GtkComboBox or %NULL
  **/
 GtkComboBox *
 go_gtk_builder_combo_box_init_text (GtkBuilder *gui, char const *widget_name)
@@ -972,7 +972,6 @@ save_info_state_free (SaveInfoState *state)
 static void
 cb_format_combo_changed (GtkComboBox *combo, SaveInfoState *state)
 {
-	GOImageFormatInfo const *format_info;
 	gboolean sensitive;
 	int i;
 
@@ -1571,8 +1570,6 @@ go_menu_position_below (GtkMenu  *menu,
 	GtkWidget *widget = GTK_WIDGET (user_data);
 	gint sx, sy;
 	GtkRequisition req;
-	GdkScreen *screen;
-	gint monitor_num;
 	GdkRectangle monitor;
 	GdkWindow *window = gtk_widget_get_window (widget);
 	GtkAllocation size;
@@ -1598,9 +1595,18 @@ go_menu_position_below (GtkMenu  *menu,
 		*x = sx + size.width - req.width;
 	*y = sy;
 
-	screen = gtk_widget_get_screen (widget);
-	monitor_num = gdk_screen_get_monitor_at_window (screen, window);
-	gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+#ifdef HAVE_GDK_DISPLAY_GET_MONITOR_AT_WINDOW
+	gdk_monitor_get_geometry
+		(gdk_display_get_monitor_at_window
+		 (gtk_widget_get_display (widget), window),
+		 &monitor);
+#else
+	{
+		GdkScreen *screen = gtk_widget_get_screen (widget);
+		gint monitor_num = gdk_screen_get_monitor_at_window (screen, window);
+		gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+	}
+#endif
 
 	if (*x < monitor.x)
 		*x = monitor.x;
