@@ -34,33 +34,17 @@
 #include <math.h>
 #include <float.h>
 
-#ifndef DOUBLE
+// We need multiple versions of this code.  We're going to include ourself
+// with different settings of various macros.  gdb will hate us.
+#include <goffice/goffice-multipass.h>
+#ifndef SKIP_THIS_PASS
 
-#define DOUBLE double
-#define SUFFIX(_n) _n
-
-struct GOAccumulator_ {
+struct INFIX(GOAccumulator,_) {
 	GArray *partials;
 };
+
 #define ACC SUFFIX(GOAccumulator)
 
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-#include "go-accumulator.c"
-#undef DOUBLE
-#undef SUFFIX
-#define DOUBLE long double
-#define SUFFIX(_n) _n ## l
-
-struct GOAccumulatorl_ {
-	GArray *partials;
-};
-
-#endif
-#endif
-
-
-
-
 gboolean
 SUFFIX(go_accumulator_functional) (void)
 {
@@ -70,9 +54,6 @@ SUFFIX(go_accumulator_functional) (void)
 /**
  * go_accumulator_start: (skip)
  **/
-/**
- * go_accumulator_startl: (skip)
- **/
 void *
 SUFFIX(go_accumulator_start) (void)
 {
@@ -88,9 +69,6 @@ SUFFIX(go_accumulator_end) (void *state)
 /**
  * go_accumulator_new: (skip)
  **/
-/**
- * go_accumulator_newl: (skip)
- **/
 ACC *
 SUFFIX(go_accumulator_new) (void)
 {
@@ -102,9 +80,6 @@ SUFFIX(go_accumulator_new) (void)
 /**
  * go_accumulator_free: (skip)
  **/
-/**
- * go_accumulator_freel: (skip)
- **/
 void
 SUFFIX(go_accumulator_free) (ACC *acc)
 {
@@ -117,9 +92,6 @@ SUFFIX(go_accumulator_free) (ACC *acc)
 /**
  * go_accumulator_clear: (skip)
  **/
-/**
- * go_accumulator_clearl: (skip)
- **/
 void
 SUFFIX(go_accumulator_clear) (ACC *acc)
 {
@@ -130,9 +102,6 @@ SUFFIX(go_accumulator_clear) (ACC *acc)
 /**
  * go_accumulator_add: (skip)
  **/
-/**
- * go_accumulator_addl: (skip)
- **/
 void
 SUFFIX(go_accumulator_add) (ACC *acc, DOUBLE x)
 {
@@ -168,9 +137,6 @@ SUFFIX(go_accumulator_add) (ACC *acc, DOUBLE x)
 /**
  * go_accumulator_add_quad: (skip)
  **/
-/**
- * go_accumulator_add_quadl: (skip)
- **/
 void
 SUFFIX(go_accumulator_add_quad) (ACC *acc, const SUFFIX(GOQuad) *x)
 {
@@ -184,9 +150,6 @@ SUFFIX(go_accumulator_add_quad) (ACC *acc, const SUFFIX(GOQuad) *x)
 /**
  * go_accumulator_value: (skip)
  **/
-/**
- * go_accumulator_valuel: (skip)
- **/
 DOUBLE
 SUFFIX(go_accumulator_value) (ACC *acc)
 {
@@ -200,3 +163,11 @@ SUFFIX(go_accumulator_value) (ACC *acc)
 
 	return sum;
 }
+
+/* ------------------------------------------------------------------------- */
+
+// See comments at top
+#endif // SKIP_THIS_PASS
+#if INCLUDE_PASS < INCLUDE_PASS_LAST
+#include __FILE__
+#endif
