@@ -25,41 +25,22 @@
 #include <goffice/goffice.h>
 #include <math.h>
 
-
-#ifndef DOUBLE
+// We need multiple versions of this code.  We're going to include ourself
+// with different settings of various macros.  gdb will hate us.
+#include <goffice/goffice-multipass.h>
+#ifndef SKIP_THIS_PASS
 
 #define QUAD SUFFIX(GOQuad)
 #define QQR SUFFIX(GOQuadQR)
 #define QMATRIX SUFFIX(GOQuadMatrix)
 
-#define DOUBLE double
-#define SUFFIX(_n) _n
-
-struct GOQuadQR_ {
+struct INFIX(GOQuadQR,_) {
 	QMATRIX *V;
 	QMATRIX *R;
 	int qdet;
 };
 
 
-#ifdef GOFFICE_WITH_LONG_DOUBLE
-#include "go-matrix.c"
-#undef DOUBLE
-#undef SUFFIX
-#define DOUBLE long double
-#define SUFFIX(_n) _n ## l
-
-struct GOQuadQRl_ {
-	QMATRIX *V;
-	QMATRIX *R;
-	int qdet;
-};
-
-#endif
-
-#endif
-
-
 /**
  * go_quad_matrix_new: (skip)
  * @m: number of rows
@@ -67,13 +48,6 @@ struct GOQuadQRl_ {
  *
  * Returns: a new zero matrix.
  **/
-/**
- * go_quad_matrix_newl: (skip)
- * @m: number of rows
- * @n: number of columns
- *
- * Returns: a new zero matrix.
- **/
 QMATRIX *
 SUFFIX(go_quad_matrix_new) (int m, int n)
 {
@@ -112,12 +86,6 @@ SUFFIX(go_quad_matrix_free) (QMATRIX *A)
  *
  * Returns: a new matrix.
  **/
-/**
- * go_quad_matrix_dupl: (skip)
- * @A: Matrix to duplicate
- *
- * Returns: a new matrix.
- **/
 QMATRIX *
 SUFFIX(go_quad_matrix_dup) (const QMATRIX *A)
 {
@@ -136,13 +104,6 @@ SUFFIX(go_quad_matrix_dup) (const QMATRIX *A)
  *
  * Copies B to A.
  **/
-/**
- * go_quad_matrix_copyl:
- * @A: (out): Destination matrix.
- * @B: (transfer none): Source matrix.
- *
- * Copies B to A.
- **/
 void
 SUFFIX(go_quad_matrix_copy) (QMATRIX *A, const QMATRIX *B)
 {
@@ -167,13 +128,6 @@ SUFFIX(go_quad_matrix_copy) (QMATRIX *A, const QMATRIX *B)
  *
  * Transposes B into A.
  **/
-/**
- * go_quad_matrix_transposel:
- * @A: (out): Destination matrix.
- * @B: (transfer none): Source matrix.
- *
- * Transposes B into A.
- **/
 void
 SUFFIX(go_quad_matrix_transpose) (QMATRIX *A, const QMATRIX *B)
 {
@@ -200,14 +154,6 @@ SUFFIX(go_quad_matrix_transpose) (QMATRIX *A, const QMATRIX *B)
  *
  * Multiplies A*B and stores the result in C.
  **/
-/**
- * go_quad_matrix_multiplyl:
- * @C: (out): Destination matrix.
- * @A: Source matrix.
- * @B: Source matrix.
- *
- * Multiplies A*B and stores the result in C.
- **/
 void
 SUFFIX(go_quad_matrix_multiply) (QMATRIX *C,
 				 const QMATRIX *A,
@@ -245,17 +191,6 @@ SUFFIX(go_quad_matrix_multiply) (QMATRIX *C,
  * indicating a matrix that cannot be inverted.  (Note: this doesn't actually
  * use the eigenvalues of A, but of A after an orthogonal transformation.)
  **/
-
-/**
- * go_quad_matrix_inversel: (skip)
- * @A: Source matrix.
- * @threshold: condition number threshold.
- *
- * Returns: The inverse matrix of A.  If any eigenvalues divided by the largest
- * eigenvalue is less than or equal to the given threshold, %NULL is returned
- * indicating a matrix that cannot be inverted.  (Note: this doesn't actually
- * use the eigenvalues of A, but of A after an orthogonal transformation.)
- **/
 QMATRIX *
 SUFFIX(go_quad_matrix_inverse) (const QMATRIX *A, DOUBLE threshold)
 {
@@ -352,14 +287,6 @@ SUFFIX(go_quad_matrix_determinant) (const QMATRIX *A, QUAD *res)
  *
  * Returns: @A's pseudo-inverse.
  **/
-
-/**
- * go_quad_matrix_pseudo_inversel: (skip)
- * @A: An arbitrary matrix.
- * @threshold: condition number threshold.
- *
- * Returns: @A's pseudo-inverse.
- **/
 QMATRIX *
 SUFFIX(go_quad_matrix_pseudo_inverse) (const QMATRIX *A, DOUBLE threshold)
 {
@@ -507,19 +434,6 @@ out:
  *
  * This function solves the triangular system RT*x=b.
  **/
-/**
- * go_quad_matrix_fwd_solvel:
- * @R: An upper triangular matrix.
- * @x: (out): Result vector.
- * @b: Input vector.
- * @allow_degenerate: If %TRUE, then degenerate dimensions are ignored other
- * than being given a zero result.  A degenerate dimension is one whose
- * diagonal entry is zero.
- *
- * Returns: %TRUE on error.
- *
- * This function solves the triangular system RT*x=b.
- **/
 gboolean
 SUFFIX(go_quad_matrix_fwd_solve) (const QMATRIX *R, QUAD *x, const QUAD *b,
 				  gboolean allow_degenerate)
@@ -573,19 +487,6 @@ SUFFIX(go_quad_matrix_fwd_solve) (const QMATRIX *R, QUAD *x, const QUAD *b,
  *
  * This function solves the triangular system R*x=b.
  **/
-/**
- * go_quad_matrix_back_solvel:
- * @R: An upper triangular matrix.
- * @x: (out): Result vector.
- * @b: Input vector.
- * @allow_degenerate: If %TRUE, then degenerate dimensions are ignored other
- * than being given a zero result.  A degenerate dimension is one whose
- * diagonal entry is zero.
- *
- * Returns: %TRUE on error.
- *
- * This function solves the triangular system R*x=b.
- **/
 gboolean
 SUFFIX(go_quad_matrix_back_solve) (const QMATRIX *R, QUAD *x, const QUAD *b,
 				   gboolean allow_degenerate)
@@ -632,12 +533,6 @@ SUFFIX(go_quad_matrix_back_solve) (const QMATRIX *R, QUAD *x, const QUAD *b,
  * @emin: (out): Smallest absolute eigen value.
  * @emax: (out): Largest absolute eigen value.
  **/
-/**
- * go_quad_matrix_eigen_rangel:
- * @A: Triangular matrix.
- * @emin: (out): Smallest absolute eigen value.
- * @emax: (out): Largest absolute eigen value.
- **/
 void
 SUFFIX(go_quad_matrix_eigen_range) (const QMATRIX *A,
 				    DOUBLE *emin, DOUBLE *emax)
@@ -693,13 +588,6 @@ SUFFIX(go_quad_matrix_dump) (const QMATRIX *A, const char *fmt)
  * of R, simply add m-n null rows.)
  * Returns: (transfer full): a new #GOQuadQR.
  **/
-
-/**
- * go_quad_qr_newl: (skip)
- * @A: input matrix
- *
- * Returns: (transfer full): qr decomposition.
- **/
 QQR *
 SUFFIX(go_quad_qr_new) (const QMATRIX *A)
 {
@@ -822,13 +710,6 @@ SUFFIX(go_quad_qr_determinant) (const QQR *qr, QUAD *det)
  * Returns: the small R from the decomposition, i.e., a square matrix
  * of size n.  To get the large R, if needed, add m-n zero rows.
  **/
-/**
- * go_quad_qr_rl:
- * @qr: A QR decomposition.
- *
- * Returns: the small R from the decomposition, i.e., a square matrix
- * of size n.  To get the large R, if needed, add m-n zero rows.
- **/
 const QMATRIX *
 SUFFIX(go_quad_qr_r) (const QQR *qr)
 {
@@ -844,13 +725,6 @@ SUFFIX(go_quad_qr_r) (const QQR *qr)
  *
  * Replaces @x by Q^t * x
  **/
-/**
- * go_quad_qr_multiply_qtl:
- * @qr: A QR decomposition.
- * @x: (inout): a vector.
- *
- * Replaces @x by Q^t * x
- **/
 void
 SUFFIX(go_quad_qr_multiply_qt) (const QQR *qr, QUAD *x)
 {
@@ -881,14 +755,6 @@ SUFFIX(go_quad_qr_multiply_qt) (const QQR *qr, QUAD *x)
  * Marks dimension i of the qr decomposition as degenerate.  In practice
  * this means setting the i-th eigenvalue of R to zero.
  **/
-/**
- * go_quad_qr_mark_degeneratel: (skip)
- * @qr: A QR decomposition.
- * @i: a dimension
- *
- * Marks dimension i of the qr decomposition as degenerate.  In practice
- * this means setting the i-th eigenvalue of R to zero.
- **/
 void
 SUFFIX(go_quad_qr_mark_degenerate) (QQR *qr, int i)
 {
@@ -897,3 +763,11 @@ SUFFIX(go_quad_qr_mark_degenerate) (QQR *qr, int i)
 
 	qr->R->data[i][i] = SUFFIX(go_quad_zero);
 }
+
+/* ------------------------------------------------------------------------- */
+
+// See comments at top
+#endif // SKIP_THIS_PASS
+#if INCLUDE_PASS < INCLUDE_PASS_LAST
+#include __FILE__
+#endif
