@@ -434,7 +434,7 @@ gog_box_plot_class_init (GogPlotClass *gog_plot_klass)
 	plot_klass->desc.num_series_max = G_MAXINT;
 	plot_klass->series_type = gog_box_plot_series_get_type ();
 	plot_klass->axis_set = GOG_AXIS_SET_XY;
-	plot_klass->desc.series.style_fields	= GO_STYLE_LINE | GO_STYLE_FILL;
+	plot_klass->desc.series.style_fields	= GO_STYLE_LINE | GO_STYLE_OUTLINE | GO_STYLE_FILL;
 	plot_klass->axis_get_bounds   		= gog_box_plot_axis_get_bounds;
 }
 
@@ -468,6 +468,7 @@ gog_box_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 	GSList *ptr;
 	GOStyle *style;
 	int num_ser = 1;
+	gboolean show_low_whisker, show_high_whisker;
 
 	area = gog_chart_view_get_plot_area (view->parent);
 	chart_map = gog_chart_map_new (chart, area,
@@ -554,6 +555,8 @@ gog_box_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 			min = series->vals[0];
 			max = series->vals[4];
 		}
+		show_low_whisker = min < series->vals[1];
+		show_high_whisker = max > series->vals[3];
 		min = gog_axis_map_to_view (map, min);
 		qu1 = gog_axis_map_to_view (map, series->vals[1]);
 		med = gog_axis_map_to_view (map, series->vals[2]);
@@ -567,14 +570,20 @@ gog_box_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 			go_path_close (path);
 			gog_renderer_draw_shape (view->renderer, path);
 			go_path_clear (path);
-			go_path_move_to (path, y + hbar, min);
-			go_path_line_to (path, y - hbar, min);
-			go_path_move_to (path, y + hbar, max);
-			go_path_line_to (path, y - hbar, max);
-			go_path_move_to (path, y, max);
-			go_path_line_to (path, y, qu3);
-			go_path_move_to (path, y, min);
-			go_path_line_to (path, y, qu1);
+			/* whiskers are displayed only if they are not inside the quartile box
+			 * which might be the case if outliers are taken into account, see #345 */
+			if (show_low_whisker) {
+				go_path_move_to (path, y + hbar, min);
+				go_path_line_to (path, y - hbar, min);
+				go_path_move_to (path, y, min);
+				go_path_line_to (path, y, qu1);
+			}
+			if (show_high_whisker) {
+				go_path_move_to (path, y + hbar, max);
+				go_path_line_to (path, y - hbar, max);
+				go_path_move_to (path, y, max);
+				go_path_line_to (path, y, qu3);
+			}
 			go_path_move_to (path, y - hrect, med);
 			go_path_line_to (path, y + hrect, med);
 			gog_renderer_stroke_shape (view->renderer, path);
@@ -587,14 +596,18 @@ gog_box_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 			go_path_close (path);
 			gog_renderer_draw_shape (view->renderer, path);
 			go_path_clear (path);
-			go_path_move_to (path, min, y + hbar);
-			go_path_line_to (path, min, y - hbar);
-			go_path_move_to (path, max, y + hbar);
-			go_path_line_to (path, max, y - hbar);
-			go_path_move_to (path, max, y);
-			go_path_line_to (path, qu3, y);
-			go_path_move_to (path, min, y);
-			go_path_line_to (path, qu1, y);
+			if (show_low_whisker) {
+				go_path_move_to (path, min, y + hbar);
+				go_path_line_to (path, min, y - hbar);
+				go_path_move_to (path, min, y);
+				go_path_line_to (path, qu1, y);
+			}
+			if (show_high_whisker) {
+				go_path_move_to (path, max, y + hbar);
+				go_path_line_to (path, max, y - hbar);
+				go_path_move_to (path, max, y);
+				go_path_line_to (path, qu3, y);
+			}
 			go_path_move_to (path, med, y - hrect);
 			go_path_line_to (path, med, y + hrect);
 			gog_renderer_stroke_shape (view->renderer, path);
